;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Plotter Queuing utility for HPGL/2 Automatic Batch Plotting
;; AUTHOR: BILL KRAMER
;; REVISED BY: JIM ARTHUR 10/94
;; QPLT.lsp ver 1.2
;;
;; History:  10/20/94, added billable/ non-billable dialog
;;           10/26/94, added C:#dwg function to return to current drawing.
;;                     added capability to make half-size plots.
;;                     added system variable dbmod to answer yes to
;;                     acad, quit "yes"
;;                     pen tables added for half or full size plots.
;;                     NOTE: you must add these files to your acad path.
;;This verison has been tested only on HP 7600 color electrostatic.
;;This version is an example for the modification and or additions
;;to the program written by Bill Kramer.
;;This program is public domain. You may modify it as you wish.
;;Neither William Kramer or James Arthur warrents the contents.
;;You must use this code at your own risk. Enjoy.
;;================================================================
;; Listing 1:  Plotter Queuing Utility
;;
(defun C:QPLT2 ( / PQ FH DH INX DFL)
  (setq DH (load_dialog "qplt2.DCL")
        DFL ;;;default list
          (list ;;empty drawing name
                "1"   ;;To fit flag
                "1.0" ;;scale 1
                "1.0" ;;scale 2
                "0"   ;;rotation
                "0.0" ;;origin x
                "0.0" ;;origin y
                "L"   ;;plot display
                "AA"  ;;plot in autocad; default not freeplot
                "BL"  ;;Charge plots to job; default billable
                "FSS" ;;Full size plot
          )
  )
  ;; Run dialog, loading PQ list
  (if (and DH (new_dialog "QPLOT2" DH)) ;name of qplt2 dialog (see qplt2.dcl)
    (progn
      (QP:ACT) ;;define dialog actions
      ;; If ok return from dialog, write script file
      (if (= (start_dialog) 1)
        (progn
           (setq FH (open "d:\\acad\\support\\qplt.SCR" "w")) ;; open plotter queue script
           ;this will open a script file by the above name. You may change to suit your
           ;drive, dir, and name. ie., "c:\\script\\qplt.scr" "w"
           (mapcar 'qp:build pq)


           (if (= k 1);if Autocad, not freeplot
           (c:#dwg);do listing 2a.
           )

           (setq FH (close FH))
        )
      )
      (unload_dialog DH)
    )
    (prompt "\nUnable to load qplt2.DCL file")
  )
  (princ)
    (IF (= k 1);if autocad, not freeplot,
       (progn (command "script" "d:/acad/support/qplt"))) ;execute the script file if autocad
      ;change this script file to any name on your Acad path.
);defun
(princ)
;;================================================================
;; Listing 2.
;; QP:ACT  define dialog box actions
;;
(defun QP:ACT ()
  (action_tile "DWG" "(QP:UPDT (setq INX (atoi $value)))")
  (action_tile "SDWG" "(QP:SD)") ;;select new file w/getfiled
  (action_tile "DDWG" "(QP:RD)") ;;delete current file from list
  (action_tile "EDWG" "(QP:AL $value)") ;;add user input
  (action_tile "XS" "(QP:SUB INX 2 $value)") ;;edit scale1
  (action_tile "YS" "(QP:SUB INX 3 $value)") ;;edit scale2
  (action_tile "FS" "(QP:SUB INX 1 $value)") ;;edit fit flag
  (action_tile "R0" "(QP:SUB INX 4 \"0\")")  ;;edit rotation
  (action_tile "R90" "(QP:SUB INX 4 \"90\")")
  (action_tile "R180" "(QP:SUB INX 4 \"180\")")
  (action_tile "R270" "(QP:SUB INX 4 \"270\")")
  (action_tile "OX" "(QP:SUB INX 5 $value)") ;;origin X
  (action_tile "OY" "(QP:SUB INX 6 $value)") ;;       Y
  (action_tile "D" "(QP:SUB INX 7 $key)") ;; Display
  (action_tile "E" "(QP:SUB INX 7 $key)") ;; Extents
  (action_tile "L" "(QP:SUB INX 7 $key)") ;; Limits
 ; (action_tile "W" "(QP:SUB INX 7 $key)") ;; Window;;;disabled

  ;; plot modes
  (action_tile "AA" "(setq K 1)") ;; Autocad
  (action_tile "FF" "(setq K 2)") ;; Freeplot
  ;; accounting information
  (action_tile "BL" "(setq ACT 3)") ;; billable
  (action_tile "NB" "(setq ACT 4)") ;; nonbillable
  ;;
    ;; accounting information
  (action_tile "FSS" "(setq PLO 5)") ;; Full size plots
  (action_tile "HSS" "(setq PLO 6)") ;; Half size plots
  ;;
  ;; Set default values into dialog box
  ;;
  (set_tile "L" "1") ;;limits
  (set_tile "OX" "0.0")
  (set_tile "OY" "0.0")
  (set_tile "R0" "1")
  (set_tile "FS" "1")
  (set_tile "XS" "1.0")
  (set_tile "YS" "1.0")
  (set_tile "AA" "1");;Autocad
  (set_tile "BL" "1");;Charge plots to job
  (set_tile "FSS" "1");;Full size plots
  ;;
  (mode_tile "PARAMS" 1)
  (mode_tile "DDWG" 1)
  ;;
  ;;set up other plotting variables
  (setq k 1
   pf (getvar "dimscale")
   ACT 3  ;action to bill plots
   pltsx (car(getvar"limmax"))
   pltsx (/ pltsx pf)
   pltsy (cadr(getvar"limmax"))
   pltsy (/ pltsy pf)
   size (strcat(rtos pltsx 2 2)","(rtos(+ pltsy 0.5)2 2);make window size
  );setq
   );defun QP:ACT
;;================================================================
;; Listing 2a.
;; C:#DWG  Get and format the name of the current drawing to return to
(defun c:#dwg ();(/ #dname)
(setvar "cmdecho" 0)
(setq
  #dname (getvar "dwgname")
)
(while(=(getvar "dwgname")
 (progn
  (if(/=(substr #dname 1 3)(substr(getvar "dwgprefix")1 3))
   (setq #dname (strcat(getvar "dwgprefix")#dname)))
     )
    )
  )
   (write-line ".OPEN" FH)
   (write-line "(if (/= (getvar \"dbmod\") 0) (write-line \"y\"))" FH)
   (write-line #dname FH)
);defun C:#DWG
;;================================================================
;; Listing 3.
;; QP:SUB (index sub-index newvalue)
;;   side-effect, sets global PQ list
;;
(defun QP:SUB (IX JX NV / TMP)
  (setq TMP (nth IX PQ)
        TMP (nsub JX NV TMP)
        PQ (subst TMP (nth IX PQ) PQ)
  )
)
;;Substitute IIth member of LST with NEW value
(defun NSUB (II NEW LST)
  (cond
    ((= II 0) (cons NEW (cdr LST)))
    (t (cons (car LST) (NSUB (1- II) NEW (cdr LST))))
  )
)
;;================================================================
;; Listing 4.
;; QP:SD add file to list based on GETFILED
;;
(defun QP:SD ()
  (setq TMP (getfiled "Select drawing file" "" "dwg" 0))
  (QP:AL TMP)
)
;;----------------------------------------------------------------
(defun QP:AL (TMP)
  (if (and TMP
           (not (assoc TMP PQ))
           (or (findfile TMP) (findfile (strcat TMP ".dwg"))))
    (progn
      (if PQ
        (setq PQ (append PQ (list (cons TMP (cdr (last PQ)))))
              INX (1+ INX))
        (progn
          (setq PQ (list (cons TMP DFL))
                INX 0)
          (mode_tile "PARAMS" 0) ;;enable dialog
          (mode_tile "DDWG" 0)
        )
      )
      (start_list "DWG")
      (mapcar 'add_list (mapcar 'car PQ))
      (end_list)
      (QP:UPDT INX)
    )
    (alert "Drawing name not accepted")
  )
)
;;-------------------------------------------------------------
(defun QP:UPDT (INX / TMP)
  (setq TMP (nth INX PQ))
  (set_tile "DWG" (itoa INX))
  (set_tile "FS" (nth 1 TMP))
  (set_tile "XS" (nth 2 TMP))
  (set_tile "YS" (nth 3 TMP))
  (set_tile "OX" (nth 5 TMP))
  (set_tile "OY" (nth 6 TMP))
  (set_tile (strcat "R" (nth 4 TMP)) "1")
  (set_tile (nth 7 TMP) "1")
)
;;===========================================================
;; Listing 5.
;; QP:RD remove current member from list
;;
(defun QP:RD ( / CNT TMP)
  (if (= (length PQ) 1)
    (progn
      (setq PQ nil)
      (mode_tile "PARAMS" 1)
      (mode_tile "DDWG" 1)
      (start_list "DWG")
      (end_list)
    )
    (progn
      (setq CNT 0)
      (repeat (length PQ)
        (if (/= CNT INX)
          (setq TMP (cons (nth CNT PQ) TMP)))
        (setq CNT (1+ CNT))
      )
      (setq PQ (reverse TMP))
      (if (>= INX (length PQ))
        (setq INX (1- (length PQ))))
      (start_list "DWG")
      (mapcar 'add_list (mapcar 'car PQ))
      (end_list)
      (QP:UPDT INX)
    )
  )
)
;;================================================================
;; Listing 6
;; QP:BUILD  output script portion to plot drawing
;;
;; D_REC (<dwg> <Fit> <Scale1> <Scale2> <Rot> <OrX> <OrY> <What>)
;;          0     1      2        3       4     5     6      7
;;
;;
;;
(defun QP:BUILD (D_REC)

(if (= plo 5)
  (progn
   (setq table (strcat "D:/acad/support/" "std-full.ptb"))))

(if (= plo 6)
  (progn
   (setq table (strcat "D:/acad/support/" "std-half.ptb"))
   (setq pltsxh (* pltsx 0.5)) ;change to half of pltsx
   (setq pltsyh (* pltsy 0.5)) ;change to half of pltsy
   (setq pltsxh nil)
   (setq pltsx nil)
  ; (setq hsize (strcat(rtos pltsx 2 2)","(rtos(+ pltsy 0.5)2 2)));make window size
    (setq size (strcat(rtos pltsxh 2 2)","(rtos(+ pltsyh 0.5)2 2)));make window size
   );if
   );progn

 (if (not TABLE)
  (progn
   (setq table (strcat "D:/acad/support/" "std-full.ptb"))))

(if (= k 1)
    (PROGN
  (write-line ".OPEN" FH)
  (write-line "(if (/= (getvar \"dbmod\") 0) (write-line \"y\"))" FH)
  (write-line (nth 0 D_REC) FH)
;  (write-line "stamp" FH);date stamp
  (write-line "_.PLOT" FH)
    );progn
);if
  (progn
(IF (= k 2)
  (progn
    (write-line "_.plot" FH)
    (write-line (nth 0 D_REC) FH) ;selected drawing name

;    (write-line "stamp" FH);date stamp
);if
  );progn
  (write-line (nth 7 D_REC) FH) ;what to plot VIEW,EXTENTS,DISPLAY
  ;;====================================================
  ;;IMPORTANT! THE FOLLOWING IS HPGL/2 SPECIFIC!
  ;;the following will vary depending on the driver you are using.
  ;;change to suit (note this part only writes the calls to the script file.
  ;;change this to the order of how your plotter asks for instructions.
  (write-line "Y" FH) ;CHANGE SOME PARAMETERS ;;;Ch
  (write-line "N" FH) ;Change plotter         ;;;Ch
  (write-line "1" FH) ;Number of copies       ;;;Ch
  (write-line "y" FH) ;change pen settings    ;;;Ch
   (setq tbl(open TABLE "r"))
   (repeat 16(princ (strcat(read-line tbl)"   ") FH)) ;;read pen setting table
   (close tbl)
  (write-line "x" fh) ;exit                   ;;;Ch
  (write-line "y" fh) ;write to a file        ;;;Ch
  (write-line "i" fh) ;inch units             ;;;Ch
  (write-line (strcat (nth 5 D_REC) "," (nth 6 D_REC)) FH) ;origin  ;;;Ch
  (write-line size FH) ;plot size             ;;;Ch
  (write-line (nth 4 D_REC) FH) ;rotation     ;;;Ch
  (write-line "N" FH) ;remove hidden lines    ;;;Ch
  (if (= (nth 1 D_REC) "1");;fixed flag       ;;;Ch
  (write-line "F" FH);fixed                   ;;;Ch
  (write-line (strcat (nth 2 D_REC) "=" (nth 3 D_REC)) FH) ;;;Ch
  )
  (write-line "lpt1" FH) ;enter file name for .plt, or lpt1, lpt2 for device
  (write-line "delay 5000" FH)
  )
;);was end
(if(= ACT 3) ; this section can be commented out all together if you
;don't want or need to track your plots for acounting.
 (progn
    (setq pfnm(open "e:/plotlist/plot-lst.lst" "a"));change this line
    ;to any drive, dir, file, to suit.
    (setq dn (nth 0 D_REC));SELECTED DWG NAME
    ;(setq dn(getvar"dwgname"))
    (setq l(strlen dn))
    (while (not(= "\\" (substr dn l 1)))(setq l(1- l)))
    (setq dname(substr dn(1+ l)))

    (setq job(substr(getvar"dwgprefix")4))
    (setq jl 5)
    (while (not(= "\\"(substr job jl 1)))(setq jl (1+ jl)))
    (setq jobno(substr job 1 (1- jl)))

    (setq DATT(rtos(getvar"CDATE")2 4))
    (setq date(strcat(substr datt 5 2)"-"(substr datt 7 2)"-"(substr datt 3 2)))
    (setq time(strcat(substr datt 10 2)":"(substr datt 12)))
    (setq nm (getvar "loginname"))
    (princ jobno pfnm)
    (princ "," pfnm)
    (princ dname pfnm)
    (princ "," pfnm)
    (princ date pfnm)
    (princ "," pfnm)
    (princ time pfnm)
    (princ "," pfnm)
    (princ nm pfnm)
    (princ "," pfnm)
    (progn
    (if(= plo 5)
    (progn                  ;(princ "LARGE" pfnm))
    (if(<= pltsx 17.0)(princ "SMALL" pfnm)
    (princ "LARGE" pfnm))
    );if
    );progn
    (if(= plo 6)
    (princ "-Halfsize" pfnm))
    );if
    (princ "\n" pfnm)
    (close pfnm)
 ));plot-lst
 );if act 3
);defun -end of program